library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.all;

entity main is
	port(
		BTN       : in  std_logic_vector(1 downto 0);
		NRES      : in  std_logic;
		LED       : out std_logic_vector(3 downto 0);
		SEG       : out std_logic_vector(7 downto 0);
		DISP      : out std_logic_vector(3 downto 0);
		CLK_10MHz : in  std_logic;
		WS2812B   : out std_logic
	);
end main;

architecture logic of main is
	signal bcd_number  : std_logic_vector(15 downto 0);
	signal clk_1kHz    : std_logic;
	signal clk_10Hz    : std_logic;
	signal clk_50MHz   : std_logic;
	signal act         : std_logic;
	signal min_int     : std_logic_vector(5 downto 0);
	signal sec_int     : std_logic_vector(5 downto 0);
	signal dots        : std_logic_vector(3 downto 0);
	signal display_num : std_logic_vector(15 downto 0);
	signal nseg        : std_logic_vector(7 downto 0);
	signal toggle      : std_logic;
	signal toggle_n    : std_logic;
	signal rst         : std_logic;
	signal nrst        : std_logic;
	signal working     : std_logic;
begin
	dots   <= "101" & (not act);
	LED(0) <= not toggle;
	LED(1) <= not rst;
	LED(3) <= '0';

	btn1 : entity pb_debouncer port map(CLK_10MHz, btn(1), toggle_n);
	btn2 : entity pb_debouncer port map(CLK_10MHz, nres, nrst);
	rst    <= not nrst;
	toggle <= not toggle_n;

	md : entity work.multi_display port map(bcd_number, dots, clk_1kHz, SEG, DISP);
	decoder_min : entity work.dec2bcd port map(min_int, bcd_number(15 downto 8));
	decoder_sec : entity work.dec2bcd port map(sec_int, bcd_number(7 downto 0));
	div1kHz : entity divider generic map(10000) port map(CLK_10MHz, '0', clk_1kHz);
	div10Hz : entity divider generic map(100) port map(clk_1kHz, '0', clk_10Hz);
	stp : entity work.pomodoro port map(clk_10Hz, rst, toggle, min_int, sec_int, act, working);

	clk50Mhz : entity PLL_50 port map(CLK_10MHz, clk_50MHz);
	leddrv : entity work.WS2812B port map(
			R           => (7 => working, others => '0'),
			G           => (7 => not working, others => '0'),
			B           => (others => '0'),
			CLOCK       => clk_50MHz,
			DATA_OUT    => WS2812B,
			FORCE_RESET => '0');
end;
